@@ -4,6 +4,8 @@ from plone.app.content.browser.tableview import Table
 from plone.app.content.browser.tableview import TableBrowserView
 from plone.registry.interfaces import IRegistry
 from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone.utils import human_readable_size
+from Products.CMFPlone.utils import isExpired
 from Products.CMFPlone.utils import safe_unicode
 from six.moves import map
 from six.moves.urllib.parse import quote_plus
@@ -16,7 +18,12 @@ from zope.publisher.browser import BrowserView
 class FullReviewListView(BrowserView):
 
     def revlist(self):
-        return self.context.my_worklist()
+        portal_membership = getToolByName(self.context, 'portal_membership')
+        portal_workflow = getToolByName(self.context, 'portal_workflow')
+        if portal_membership.isAnonymousUser():
+            return []
+
+        return portal_workflow.getWorklistsResults()
 
     def url(self):
         return self.context.absolute_url() + '/full_review_list'
@@ -53,15 +60,21 @@ class ReviewListTable(object):
                                        name=u'plone_layout')
         portal_workflow = getToolByName(self.context, 'portal_workflow')
         portal_types = getToolByName(self.context, 'portal_types')
+        portal_membership = getToolByName(self.context, 'portal_membership')
 
         registry = getUtility(IRegistry)
         use_view_action = registry.get(
             'plone.types_use_view_action_in_listings', ())
 
-        browser_default = self.context.browserDefault()
+        browser_default = plone_utils.browserDefault(self.context)
 
         results = list()
-        for i, obj in enumerate(self.context.my_worklist()):
+        if portal_membership.isAnonymousUser():
+            worklist = []
+        else:
+            worklist = portal_workflow.getWorklistsResults()
+
+        for i, obj in enumerate(worklist):
             if i % 2 == 0:
                 table_row_class = "even"
             else:
@@ -114,7 +127,7 @@ class ReviewListTable(object):
                 title_or_id=obj.pretty_title_or_id(),
                 description=obj.Description(),
                 obj_type=obj.Type,
-                size=obj.getObjSize(),
+                size=human_readable_size(obj.get_size()),
                 modified=modified,
                 type_class=type_class,
                 wf_state=review_state,
@@ -126,7 +139,7 @@ class ReviewListTable(object):
                 relative_url=relative_url,
                 view_url=view_url,
                 table_row_class=table_row_class,
-                is_expired=self.context.isExpired(obj)
+                is_expired=isExpired(obj)
             ))
         return results
 
