@@ -9,6 +9,7 @@ from plone.app.layout.navigation.root import getNavigationRoot
 from plone.app.querystring import queryparser
 from plone.app.z3cform.interfaces import IFieldPermissionChecker
 from plone.autoform.interfaces import WRITE_PERMISSIONS_KEY
+from plone.memoize.view import memoize
 from plone.supermodel.utils import mergedTaggedValueDict
 from Products.CMFCore.utils import getToolByName
 from Products.CMFPlone import PloneMessageFactory as _
@@ -16,7 +17,9 @@ from Products.CMFPlone.utils import safe_unicode
 from Products.Five import BrowserView
 from Products.MimetypesRegistry.MimeTypeItem import guess_icon_path
 from Products.MimetypesRegistry.MimeTypeItem import PREFIX
+from Products.PortalTransforms.transforms.safe_html import SafeHTML
 from types import FunctionType
+from z3c.form.interfaces import IAddForm
 from z3c.form.interfaces import ISubForm
 from zope.component import getUtility
 from zope.component import queryAdapter
@@ -32,6 +35,13 @@ import itertools
 import six
 
 
+# six has no char_types, but it should:
+# bytes + string + unicode, where defined.
+if six.PY3:
+    char_types = (bytes, str)
+else:
+    char_types = basestring
+
 logger = getLogger(__name__)
 
 MAX_BATCH_SIZE = 500  # prevent overloading server
@@ -209,6 +219,7 @@ class BaseVocabularyView(BrowserView):
             attributes = attributes.split(',')
 
         translate_ignored = self.get_translated_ignored()
+        transform = SafeHTML()
         if attributes:
             base_path = self.get_base_path(context)
             sm = getSecurityManager()
@@ -232,18 +243,19 @@ class BaseVocabularyView(BrowserView):
                             val = val()
                         else:
                             continue
-                    if key == 'path':
+                    if key == 'path' and val is not None:
                         val = val[len(base_path):]
                     if (
                         key not in translate_ignored and
                         isinstance(val, six.string_types)
                     ):
-                        item[key] = translate(
+                        val = translate(
                             _(safe_unicode(val)),
                             context=self.request
                         )
-                    else:
-                        item[key] = val
+                    if isinstance(val, char_types):
+                        val = transform.scrub_html(val)
+                    item[key] = val
                     if key == 'getMimeIcon':
                         item[key] = None
                         # get mime type icon url from mimetype registry'
@@ -265,8 +277,8 @@ class BaseVocabularyView(BrowserView):
                                 )
                 items.append(item)
         else:
-            items = [{'id': item.value,
-                      'text': item.title} for item in results]
+            items = [{'id': transform.scrub_html(item.value),
+                      'text': transform.scrub_html(item.title) if item.title else ""} for item in results]
 
         if total == 0:
             total = len(items)
@@ -373,19 +385,23 @@ class SourceView(BaseVocabularyView):
             context = self.context.context
         return context
 
+    @property
+    @memoize
+    def default_permission(self):
+        if IAddForm.providedBy(self.context.form):
+            return "cmf.AddPortalContent"
+        return "cmf.ModifyPortalContent"
+
     def get_vocabulary(self):
         widget = self.context
         field = widget.field.bind(widget.context)
 
         # check field's write permission
         info = mergedTaggedValueDict(field.interface, WRITE_PERMISSIONS_KEY)
-        permission_name = info.get(field.__name__, 'cmf.ModifyPortalContent')
+        permission_name = info.get(field.__name__, self.default_permission)
         permission = queryUtility(IPermission, name=permission_name)
         if permission is None:
-            permission = getUtility(
-                IPermission,
-                name='cmf.ModifyPortalContent'
-            )
+            permission = getUtility(IPermission, name=self.default_permission)
         if not getSecurityManager().checkPermission(
             permission.title,
             self.get_context()
